/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RAST_MERGER_H
#define RAST_MERGER_H

/**
 * @file merger.h merger
 */

#include "rast/db.h"

/**
 * @defgroup merger merger
 * @{
 */

typedef struct {
    /** The super class instance */
    rast_db_t base;

    /** The number of databases */
    int num_merged_dbs;
    /** Some databases for merging */
    rast_db_t **merged_dbs;
    const rast_property_t *properties;
    int num_properties;
} rast_merger_t;

/**
 * Merge some opened databases.
 * @param db The database opened
 * @param merge_dbs Some databases for merging.
 * @param num_merge_dbs The number of databases.
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_merger_open(rast_db_t **db,
                               rast_db_t **merge_dbs, int num_merge_dbs,
                               apr_pool_t *pool);

/**
 * Please use rast_db_close().
 * @see rast_db_close()
 */
rast_error_t *rast_merger_close(rast_db_t *db);

/**
 * Please use rast_db_search().
 * @see rast_db_search()
 */
rast_error_t *rast_merger_search(rast_db_t *db,
                                 const char *query,
                                 rast_search_option_t *options,
                                 rast_result_t **result, apr_pool_t *pool);

/**
 * Please use rast_db_properties().
 * @see rast_db_properties()
 */
const rast_property_t *rast_merger_properties(rast_db_t *db,
                                              int *num_properties);

/** @} */

#endif
