/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RESULT_H
#define RESULT_H

/**
 * @file result.h result
 */

#include <apr_ring.h>

#include "rast/property.h"

typedef struct rast_term_t {
    const char *term;
    int doc_count;
    APR_RING_ENTRY(rast_term_t) link;
} rast_term_t;

APR_RING_HEAD(rast_term_ring_t, rast_term_t);
typedef struct rast_term_ring_t rast_term_ring_t;

typedef struct rast_term_frequency_t {
    int count;
    rast_pos_t pos;
    APR_RING_ENTRY(rast_term_frequency_t) link;
} rast_term_frequency_t;

APR_RING_HEAD(rast_term_frequency_ring_t, rast_term_frequency_t);
typedef struct rast_term_frequency_ring_t rast_term_frequency_ring_t;

typedef struct rast_candidate_t {
    rast_doc_id_t doc_id;
    rast_term_frequency_ring_t terms;
    APR_RING_ENTRY(rast_candidate_t) link;
} rast_candidate_t;

APR_RING_HEAD(rast_candidate_ring_t, rast_candidate_t);
typedef struct rast_candidate_ring_t rast_candidate_ring_t;

typedef struct rast_query_result_t {
    rast_term_ring_t terms;
    rast_candidate_ring_t candidates;
} rast_query_result_t;

/**
 * @defgroup result result
 * @{
 */

/** A structure that represents a term in a query */
typedef struct {
    /** The term */
    char *term;
    /** The number of documents that contains the term */
    int doc_count;
} rast_result_term_t;

#if 0
/* todo: for multiple search word summary position */
typedef struct {
    int term_no;
    rast_pos_t pos;
} rast_term_pos_t;
#endif

/** A structure that represents a result item */
typedef struct {
    /** The document ID */
    rast_doc_id_t doc_id;
    /** The database index */
    int db_index;
    /** The score of the document */
    int score;
    /** The summary of the document */
    char *summary;
    /** The length of summary in bytes */
    int summary_nbytes;
#if 0
    /* todo: for multiple search word summary position */
    rast_term_pos_t *term_positions;
    int num_term_positions;
#endif
    /** The properties of the document */
    rast_value_t *properties;
} rast_result_item_t;

/** A structure that represents a search result */
typedef struct {
    /** The number of indices in search database */
    int num_indices;
    /** The number of documents in search database */
    int num_docs;
    /** The number of documents that hit to the query */
    int hit_count;
    /** The terms in the query */
    rast_result_term_t *terms;
    /** The number of terms in the query */
    int num_terms;
    /** The items in the result */
    rast_result_item_t **items;
    /** The number of the items in the result */
    int num_items;
} rast_result_t;

/** @} */

#endif

/* vim: set filetype=c sw=4 expandtab : */
