/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RAST_TEXT_INDEX_H
#define RAST_TEXT_INDEX_H

#include <apr.h>
#include <apr_errno.h>
#include <apr_pools.h>
#include <apr_hash.h>
#include <apr_file_io.h>

#include <db.h>

#include "rast/rast.h"
#include "rast/error.h"
#include "rast/encoding.h"
#include "rast/result.h"

typedef struct rast_text_indexer_t rast_text_indexer_t;
typedef struct rast_text_index_t rast_text_index_t;

rast_error_t *rast_text_indexer_add(rast_text_indexer_t *indexer,
                                    const char *s, int len,
                                    rast_size_t *registered_chars);
rast_error_t *rast_text_indexer_commit(rast_text_indexer_t *indexer);

rast_error_t *rast_text_index_open(rast_text_index_t **index,
                                   const char *filename, int flags,
                                   rast_encoding_module_t *encoding_module,
                                   DB_ENV *db_env, int lorder,
                                   rast_size_t pos_block_size,
                                   apr_pool_t *pool);
rast_error_t *rast_text_index_close(rast_text_index_t *index);
rast_error_t *rast_text_index_register(rast_text_index_t *index,
                                       rast_doc_id_t doc_id,
                                       rast_text_indexer_t **indexer,
                                       apr_pool_t *pool);
rast_error_t *rast_text_index_sync(rast_text_index_t *index);
rast_error_t *rast_text_index_search(rast_text_index_t *index,
                                     const char *term,
                                     int need_term_frequency,
                                     rast_query_result_t **result,
                                     apr_pool_t *pool);

#endif

/* vim: set filetype=c sw=4 expandtab : */
